==> eslint-plugin-types-0.0.3.tgz: package/lib/rules/array-type.js <==
const meta = {
  docs: {
    description: 'enforce consistent array types',
    category: 'Best Practices',
    recommended: true
  },
  fixable: null,
  schema: []
};

const create = function (context) {
  const TYPE_MAP = {};

  const getScopeId = (block) =>
    `${block.type}-${block.start}`;

  const findType = (scope, id, type) => {
    if (!scope || !scope.block) {
      return null;
    }

    if (TYPE_MAP[getScopeId(scope.block)] && id in TYPE_MAP[getScopeId(scope.block)]) {
      return TYPE_MAP[getScopeId(scope.block)][id];
    }

    return type !== 'MemberExpression' && findType(scope.upper, id);
  };


  const getId = (item, full = true) => {
    if (item.type === 'VariableDeclarator') {
      return item.id.name;
    }

    const left = item.left || item.callee;

    if (left) {
      if (left.type === 'MemberExpression') {
        let name = '';
        let member = left;

        while (member.property && member.object) {
          name = `${full && member !== left ? member.property.raw : ''}${name && '.'}${name}`;
          member = member.object;
        }

        return `${member.name}${name && '.'}${name}`;
      }

      return left.name;
    }
  };

  const getType = (expression) => {
    if (!expression) return null;
    switch (expression.type) {
      case 'Literal': {
        if (expression.value === null) {
          return null
        }
        return typeof expression.value;
      }
      case 'ObjectExpression': return 'object';
      case 'ArrayExpression': return 'array';
      case 'FunctionExpression':
      case 'ArrowFunctionExpression': return 'function';
      default: return null;
    }
  };

  return {
    CallExpression (node) {
      if (!~['unshift', 'push'].indexOf(node.callee.property.name)) {
        return;
      }

      const id = getId(node);
      const scope = context.getScope();
      const declaredType = findType(scope, id, node.callee.type);

      if (!declaredType) return;

      node.arguments.forEach((argument) => {
        const newType = getType(argument);
        if (newType !== declaredType) {
          context.report({
            node: argument,
            message: `Array items must be of type ${declaredType}.`
          });
        }
      });
    },

    AssignmentExpression (node) {
      const id = getId(node, false);
      const newType = getType(node.right);

      if (!id || !newType) return null;

      const scope = context.getScope();
      const declaredType = findType(scope, id, node.left.type);

      if (!declaredType) {
        const { block } = scope;

        if (!TYPE_MAP[getScopeId(block)]) {
          TYPE_MAP[getScopeId(block)] = {};
        }

        TYPE_MAP[getScopeId(block)][id] = newType;

        return;
      }

      if (declaredType && declaredType !== newType) {
        context.report({
          node: node.right,
          message: `Can't assign type ${newType} to ${declaredType}.`,
        });
      }
    },

    ArrayExpression(node) {
      if (!~['AssignmentExpression', 'VariableDeclarator'].indexOf(node.parent.type)) {
        return;
      }

      const { block } = context.getScope();

      if (!TYPE_MAP[getScopeId(block)]) {
        TYPE_MAP[getScopeId(block)] = {};
      }

      const parent = node.parent.init || node.parent.right;
      const elements = parent.elements;
      const id = getId(node.parent);
      const newType = getType(elements[0]);

      TYPE_MAP[getScopeId(block)][id] = newType;

      elements.forEach((element) => {
        if (getType(element) !== newType) {
          context.report({
            node: element,
            message: `Array items must be of type ${newType}.`
          });
        }
      });
    }
  };
};

module.exports = {
  meta,
  create,
};

==> eslint-plugin-types-0.0.4.tgz: package/lib/rules/array-type.js <==
const { getType, getId, getScopeId, findType } = require('./helpers');

const meta = {
  docs: {
    description: 'enforce consistent array types',
    category: 'Best Practices',
    recommended: true
  },
  fixable: null,
  schema: []
};

const create = function (context) {
  const TYPE_MAP = {};

  return {
    CallExpression (node) {
      if (!~['unshift', 'push'].indexOf(node.callee.property.name)) {
        return;
      }

      const id = getId(node);
      const scope = context.getScope();
      const declaredType = findType(TYPE_MAP, scope, id, node.callee.type);

      if (!declaredType) return;

      node.arguments.forEach((argument) => {
        const newType = getType(argument);
        if (newType !== declaredType) {
          context.report({
            node: argument,
            message: `Array items must be of type ${declaredType}.`
          });
        }
      });
    },

    AssignmentExpression (node) {
      const id = getId(node, false);
      const newType = getType(node.right);

      if (!id || !newType) return null;

      const scope = context.getScope();
      const declaredType = findType(TYPE_MAP, scope, id, node.left.type);

      if (!declaredType) {
        const { block } = scope;

        if (!TYPE_MAP[getScopeId(block)]) {
          TYPE_MAP[getScopeId(block)] = {};
        }

        TYPE_MAP[getScopeId(block)][id] = newType;

        return;
      }

      if (declaredType && declaredType !== newType) {
        context.report({
          node: node.right,
          message: `Can't assign type ${newType} to ${declaredType}.`,
        });
      }
    },

    ArrayExpression(node) {
      if (!~['AssignmentExpression', 'VariableDeclarator'].indexOf(node.parent.type)) {
        return;
      }

      const { block } = context.getScope();

      if (!TYPE_MAP[getScopeId(block)]) {
        TYPE_MAP[getScopeId(block)] = {};
      }

      const parent = node.parent.init || node.parent.right;
      const elements = parent.elements;
      const id = getId(node.parent);
      const newType = getType(elements[0]);

      if (!id || !newType) return;

      TYPE_MAP[getScopeId(block)][id] = newType;

      elements.forEach((element) => {
        if (getType(element) !== newType) {
          context.report({
            node: element,
            message: `Array items must be of type ${newType}.`
          });
        }
      });
    }
  };
};

module.exports = {
  meta,
  create,
};
